@@ -3,8 +3,6 @@ import json
 import requests
 
 from cc_py_commons.config.env import app_config
-from cc_py_commons.config.pc_miler_config import PcMilerConfig
-from cc_py_commons.utils.pc_miler_utils import pc_miler_limit_reached, update_pc_miler_requests_count
 from cc_py_commons.utils.redis import location_db_conn, distance_db_conn, timezone_db_conn
 from ast import literal_eval
 
@@ -20,17 +18,6 @@ def get_location(city, state, zipcode, logger):
 
   if not data:
     logger.debug(f"geolocate.get_location - Couldn't find location: {city},{state} {zipcode} in cache.")
-    # pc_miler_can_be_called = not pc_miler_limit_reached(logger)
-    
-    # if pc_miler_can_be_called:
-    #   data = __get_pc_mile_location(city, state, zipcode, logger)
-
-  # if not data:
-    # if not pc_miler_can_be_called:
-    #   logger.debug("geolocate.get_location - Couldn't call PC Miler as we reached the permissible limits. Calling google.")
-    # else:
-    #   logger.debug(f"geolocate.get_location - PC Miler returned no result for ({city}, {state}, {zipcode}). Calling google")
-
     data = __get_google_location(city, state, zipcode, logger)
 
   if data:
@@ -54,19 +41,7 @@ def get_distance(origin_latitude, origin_longitude, destination_latitude, destin
   if distance:
     from_cache = True
 
-  if not distance:
-  #   pc_miler_can_be_called = not pc_miler_limit_reached(logger)
-    
-  #   if pc_miler_can_be_called:
-  #     distance = __get_pc_miler_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude,
-  #                                    logger, distance_cache_key)
-
-  # if not distance:
-  #   if not pc_miler_can_be_called:
-  #     logger.debug("geolocate.get_distance - Couldn't call PC Miler as we reached the permissible limits. Calling google.")
-  #   else:
-  #     logger.debug(f"geolocate.get_distance - PC Miler returned no result for {distance_cache_key}. Calling google")
-      
+  if not distance:      
     distance = __get_google_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude, logger)
 
   if distance:
@@ -101,63 +76,6 @@ def get_timezone(lat,lng, logger):
 
   return timezone
 
-def __get_pc_mile_location(city, state, zipcode, logger):
-  location = None
-  try:
-    uri = app_config.PC_MILER_URL.format("us", app_config.PC_MILER_KEY)
-
-    if city and state:
-        uri = uri + f"&city={city}&state={state}"
-
-    if zipcode:
-        uri = uri + f"&postcode={zipcode}"
-
-    response = requests.get(uri)
-    update_pc_miler_requests_count(logger)
-
-    if response:
-        locationData = json.loads(response.content)
-        
-        if locationData[0]['Address']:
-            location = {}
-            location['city'] = locationData[0]['Address']['City']
-            location['state'] = locationData[0]['Address']['State']
-            location['postcode'] = locationData[0]['Address']['Zip']
-            location['country'] = locationData[0]['Address']['CountryAbbreviation']
-            location['lat'] = float(locationData[0]['Coords']['Lat'])
-            location['lng'] = float(locationData[0]['Coords']['Lon'])
-
-  except Exception as e:
-    logger.error("geolocate.__get_pc_mile_location: Error while getting location from PC Miler", e)
-    location = None
-  return location
-
-def __get_pc_miler_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude,
-                        logger, distance_cache_key):
-  distance = None
-  try:
-    pcmiler_config = PcMilerConfig()
-    url = pcmiler_config.url + '?' # there is always at least on query parameter
-
-    for key in pcmiler_config.query_params:
-      if key == 'stops':
-        url = f"{url}&{key}={origin_longitude},{origin_latitude};{destination_longitude},{destination_latitude}"
-      elif key == 'authToken':
-        url = f"{url}&{key}={app_config.PC_MILER_KEY}"
-      elif pcmiler_config.query_params[key]:
-        url = f"{url}&{key}={pcmiler_config.query_params[key]}"
-
-    response = requests.get(url)
-    update_pc_miler_requests_count(logger)
-
-    if response:
-      distanceData = json.loads(response.content)
-      distance = distanceData[0]['TMiles']
-  except Exception as e:
-    logger.error("geolocate.__get_pc_miler_distance: Error while getting distance from PC Miler", e)
-    distance = None
-  return distance
-
 def __get_google_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude, logger):
   distance = None
   try:
@@ -192,13 +110,22 @@ def __get_google_location(city, state, zipcode, logger):
 
       loc_str = loc_str + zipcode
 
-    gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
-    response = gmaps.geocode(loc_str)
-
-    if response:
-      data = { 'city': None, 'state': None, 'postcode': None }
+    def __parse_response(response):
       components = response[0]['address_components']
+      types = response[0]['types']
       location = response[0]['geometry']['location']
+      postcode_localities = response[0].get('postcode_localities')
+
+      data = { 'city': None, 'state': None, 'postcode': None }
+      data['lat'] = location['lat']
+      data['lng'] = location['lng']
+
+      # when checking the response for a zipcode verify the city name is in the list of localities
+      if 'postal_code' in types:
+        matches = [l for l in postcode_localities if l.lower() == city.lower()]
+
+        if not  matches:
+          return None
 
       for component in components:
         if 'locality' in component['types']:
@@ -210,8 +137,25 @@ def __get_google_location(city, state, zipcode, logger):
         elif 'country' in component['types']:
           data['country'] = component['short_name']
 
-      data['lat'] = location['lat']
-      data['lng'] = location['lng']
+      return data
+
+    gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
+    response = gmaps.geocode(loc_str)
+    
+    if response:
+      if len(response) > 1:
+        logger.warning(f"Google returned multiple matches for {loc_str}")
+        return None
+
+      data = __parse_response(response)
+      
+      # For some locals like neighborhood google does not return postal_code
+      ## if zipcode is present run it only and verify against city
+      if not data['postcode'] and zipcode:
+        response = gmaps.geocode(zipcode)
+        post_code_data = __parse_response(response)
+        data['postcode'] = post_code_data['postcode']
+      
       return data
   except googlemaps.exceptions.Timeout as e:
     msg = "geolocate.__get_google_location - Location lookup timed out {0}, {1}, {2}: {3}".format(city, state, zipcode, e)
